program main
implicit none
double precision v(1000,3),fa(1000,3),fb(1000,3),x(1000,3)
double precision av(3),sv(3),dx(3)
double precision wkb,avo,sigma,e,wm,cell,tset,dt,time0,t
double precision etot,ekin,epot,u
double precision sv2,av2,cellh,aa
integer i,k,l,npart,nstep

open( 1,file='xe_ini_256.d',status='unknown')
open( 3,file='xe_out_256.d',status='unknown')
open(10,file='xe_moni_256.d',status='unknown')

wkb = 1.38065812d-23
avo = 6.02213673d+23
! read initial data
read (1,*) npart,nstep,sigma,e,wm,cell,tset,dt
  do i=1,npart
    read(1,*) x(i,1),x(i,2),x(i,3)
    read(1,*) v(i,1),v(i,2),v(i,3)
  enddo
read(1,*) time0
! remove momentum of the cell
sv=0.d0
do i=1,npart
  sv(:) = sv(:) + v(i,:)
enddo
av(:) = sv(:)/npart
do i=1,npart
  v(i,:) = v(i,:) - av(:)
enddo
! main calculation
cellh = cell * 0.5d0
call force(fb,cell,cellh,x,sigma,e,u,npart)
! main loop
do k=1,nstep
  time0 = dt + time0
  do i=1,npart
    v(i,:)=v(i,:)+dt*fb(i,:)/(2.0d0*wm)
    x(i,:)=x(i,:)+dt*v(i,:)
  enddo
! periodic boundary condition
  do i=1,npart
    do l=1,3
      dx(l)=x(i,l)-cellh
      if(abs(dx(l)).gt.cellh) x(i,l)=x(i,l)-sign(cell,dx(l))
    enddo
  enddo
  call force(fa,cell,cellh,x,sigma,e,u,npart)
  do i=1,npart
    v(i,:)=v(i,:)+dt*fa(i,:)/(2.0d0*wm)
  enddo
  fb=fa
! summation of the velocities
  sv2=0.d0
  do i=1,npart
    sv2=sv2+v(i,1)**2+v(i,2)**2+v(i,3)**2
  enddo
  av2=sv2/npart
  t=wm*av2/(3.0d0*wkb)
! calculation of the total energy
  epot = u * avo / npart
  ekin = av2 * wm / 2.0d0 * avo
  etot = epot + ekin
! monitor
  write(10,200) time0,ekin,epot,etot,t
  write(6,'(a,I6,a,3E18.9)') 'Ek ,Ep, E(',k,')=',ekin,epot,etot
  write(6,'(a,E13.5,a,F13.5)') 'time=',time0,'  T=',t
enddo
! output
write(3,400) npart,nstep,sigma,e,wm,cell,tset,dt
do i=1,npart
  write(3,200) x(i,1),x(i,2),x(i,3)
  write(3,200) v(i,1),v(i,2),v(i,3)
enddo
write(3,200) time0
  400 format(2i8,6e13.5)
  200 format(5e13.5)
end program main

! subroutine force
subroutine force(f,cell,cellh,x,sigma,e,u,npart)
implicit none
double precision f(1000,3),x(1000,3),dx(3),fdx(3)
double precision cell,cellh,sigma,e,u
double precision rc,r2,rc2,d2,rd2,rd6,rd12,fk
integer i,j,l,npart

rc=3.0d0*sigma
r2=sigma*sigma
rc2=rc*rc

u = 0.0d0
f = 0.0d0

do i=1,npart-1
  do j=i+1,npart
    do l=1,3
      dx(l)=x(i,l)-x(j,l)
      if(abs(dx(l)).gt.cellh) dx(l)=dx(l)-sign(cell,dx(l))
    enddo 
    d2=dx(1)*dx(1)+dx(2)*dx(2)+dx(3)*dx(3)
    if(d2.lt.rc2) then
      rd2=r2/d2
      rd6=rd2*rd2*rd2
      rd12=rd6*rd6
      fk=48*e*(rd12-rd6*0.5d0)
      fdx(:)=fk*dx(:)/d2
      f(i,:)=f(i,:)+fdx(:)
      f(j,:)=f(j,:)-fdx(:)
      u=u+4*e*(rd12-rd6)
    end if
  enddo
enddo
return
end
